@@ -32,13 +32,18 @@ extern "C" {
 
 struct brw_context;
 
+struct nir_shader *brw_create_nir(struct brw_context *brw,
+                                  const struct gl_shader_program *shader_prog,
+                                  const struct gl_program *prog,
+                                  gl_shader_stage stage,
+                                  bool is_scalar);
+
 void brw_setup_tex_for_precompile(struct brw_context *brw,
                                   struct brw_sampler_prog_key_data *tex,
                                   struct gl_program *prog);
 
 void brw_populate_sampler_prog_key_data(struct gl_context *ctx,
 				        const struct gl_program *prog,
-                                        unsigned sampler_count,
 				        struct brw_sampler_prog_key_data *key);
 bool brw_debug_recompile_sampler_key(struct brw_context *brw,
                                      const struct brw_sampler_prog_key_data *old_key,
@@ -54,12 +59,14 @@ brw_stage_prog_data_free(const void *prog_data);
 
 void
 brw_dump_ir(const char *stage, struct gl_shader_program *shader_prog,
-            struct gl_shader *shader, struct gl_program *prog);
+            struct gl_linked_shader *shader, struct gl_program *prog);
 
-void brw_upload_tcs_prog(struct brw_context *brw,
-                         uint64_t per_vertex_slots, uint32_t per_patch_slots);
-void brw_upload_tes_prog(struct brw_context *brw,
-                         uint64_t per_vertex_slots, uint32_t per_patch_slots);
+void brw_upload_tcs_prog(struct brw_context *brw);
+void brw_tcs_populate_key(struct brw_context *brw,
+                          struct brw_tcs_prog_key *key);
+void brw_upload_tes_prog(struct brw_context *brw);
+void brw_tes_populate_key(struct brw_context *brw,
+                          struct brw_tes_prog_key *key);
 
 #ifdef __cplusplus
 } /* extern "C" */
